// use std::iter::Peekable;
use std::str::Chars;

use color_eyre::eyre;
use eyre::eyre;


#[derive(Debug)]
enum Item {
    Value(u8),
    SubPair(Box<Pair>),
}

impl Item {
    fn magnitude(&self) -> usize {
        match self {
            Item::Value(v) => *v as usize,
            Item::SubPair(pair) => pair.magnitude(),
        }
    }
}

#[derive(Debug)]
struct Pair {
    left: Item,
    right: Item,
}

impl Pair {
    fn magnitude(&self) -> usize {
        3 * self.left.magnitude() + 2 * self.right.magnitude()
    }
}


struct Parser<'a> {
    chars: Chars<'a>,
}

impl<'a> Parser<'a> {
    fn from(src: &'a str) -> Parser<'a> {
        let mut chars = src.chars();
        chars.next(); // throw away the opening bracket so we don't nest too deeply
        Parser {chars}
    }

    fn parse(&mut self) -> eyre::Result<Pair> {
        let mut items = Vec::with_capacity(2);
        while items.len() < 2 {
            let item = match self.chars.next() {
                Some('[') => Item::SubPair(Box::new(self.parse()?)),
                Some(','|']') => {continue;},
                Some(c) => Item::Value(c.to_digit(10).unwrap() as u8),
                None => {return Err(eyre!("Ran out of characters to parse"));},
            };
            items.push(item);
        }

        let right = items.pop().unwrap();
        let left = items.pop().unwrap();
        Ok(Pair {left, right})
    }
}


fn load(data: &str) -> eyre::Result<Vec<Pair>>{
    let mut pairs = Vec::new();
    for line in data.lines() {
        let mut parser = Parser::from(line);
        let pair = parser.parse()
            .map_err(|_| eyre!("Invalid input: {}", line))?;
        pairs.push(pair);
    }
    Ok(pairs)
}


const _SAMPLE: &str = "[[[[8,7],[7,7]],[[8,6],[7,7]]],[[[0,7],[6,6]],[8,7]]]";

pub fn run(data: &str) -> eyre::Result<(usize, usize)> {
    // let pairs = load(data)?;
    // dbg!(pairs.len());
    // dbg!(&pairs[0]);

    let mut parser = Parser::from(_SAMPLE);
    let root = parser.parse()?;
    dbg!(root.magnitude());

    Ok((0, 0))
}